@@ -238,31 +238,40 @@ recode_char <- function(X, ..., default = NULL, missing = NULL, regex = FALSE,
   return(if(set) invisible(res) else res)
 }
 
-replace_na <- function(X, value = 0L, cols = NULL, set = FALSE) {
+
+na_locf <- function(x, set = FALSE) .Call(C_na_locf, x, set)
+na_focb <- function(x, set = FALSE) .Call(C_na_focb, x, set)
+
+na_locf_ph <- function(x, ph1, ph2, set = FALSE) .Call(C_na_locf, x, set)
+na_focb_ph <- function(x, ph1, ph2, set = FALSE) .Call(C_na_focb, x, set)
+
+replace_na <- function(X, value = 0L, cols = NULL, set = FALSE, type = "const") {
+  FUN <- switch(type, const =, value = scv, locf = na_locf_ph, focb = na_focb_ph,
+                stop("Unknown type:", type))
   if(set) {
     if(is.list(X)) {
       if(is.null(cols)) {
-        lapply(unattrib(X), scv, NA, value, TRUE)
+        lapply(unattrib(X), FUN, NA, value, TRUE)
       } else if(is.function(cols)) {
-        lapply(unattrib(X), function(y) if(cols(y)) scv(y, NA, value, TRUE) else y)
+        lapply(unattrib(X), function(y) if(cols(y)) FUN(y, NA, value, TRUE) else y)
       } else {
         cols <- cols2int(cols, X, attr(X, "names"), FALSE)
-        lapply(unattrib(X)[cols], scv, NA, value, TRUE)
+        lapply(unattrib(X)[cols], FUN, NA, value, TRUE)
       }
-    } else scv(X, NA, value, TRUE) # `[<-`(X, is.na(X), value = value)
+    } else FUN(X, NA, value, TRUE) # `[<-`(X, is.na(X), value = value)
     return(invisible(X))
   }
   if(is.list(X)) {
-    if(is.null(cols)) return(condalc(duplAttributes(lapply(unattrib(X), scv, NA, value), X), inherits(X, "data.table"))) # function(y) `[<-`(y, is.na(y), value = value)
+    if(is.null(cols)) return(condalc(duplAttributes(lapply(unattrib(X), FUN, NA, value), X), inherits(X, "data.table"))) # function(y) `[<-`(y, is.na(y), value = value)
     if(is.function(cols)) return(condalc(duplAttributes(lapply(unattrib(X),
-      function(y) if(cols(y)) scv(y, NA, value) else y), X), inherits(X, "data.table")))
+      function(y) if(cols(y)) FUN(y, NA, value) else y), X), inherits(X, "data.table")))
     clx <- oldClass(X)
     oldClass(X) <- NULL
     cols <- cols2int(cols, X, names(X), FALSE)
-    X[cols] <- lapply(unattrib(X[cols]), scv, NA, value) #  function(y) `[<-`(y, is.na(y), value = value)
+    X[cols] <- lapply(unattrib(X[cols]), FUN, NA, value) #  function(y) `[<-`(y, is.na(y), value = value)
     return(condalc(`oldClass<-`(X, clx), any(clx == "data.table")))
   }
-  scv(X, NA, value) # `[<-`(X, is.na(X), value = value)
+  FUN(X, NA, value) # `[<-`(X, is.na(X), value = value)
 }
 
 replace_NA <- replace_na
@@ -337,7 +346,7 @@ replace_outliers <- function(X, limits, value = NA,
    if(sl == 5L || sl == 7L) ignore.groups <- TRUE
   } else sl <- 0L
 
-  if(sl > 3L) { # Outliers according to standard deviation or MAD threashold
+  if(sl > 3L) { # Outliers according to standard deviation or MAD threshold
     if(is.list(X)) {
       if(!ignore.groups && inherits(X, c("grouped_df", "pdata.frame"))) {
         if(is.character(value)) stop("clipping is not yet supported with grouped/panel data and SDs/MADs thresholds.")
