@@ -2,7 +2,7 @@
 
 
 // Cases:
-// 0- replace_NA (only replace missing values)
+// 0- replace_na (only replace missing values)
 // 1- replace
 // 2- replace with NA rm
 // 3- demean
@@ -40,8 +40,10 @@ static inline double remainder_impl(double x, double y) {
 int TtI(SEXP x) {
   if(TYPEOF(x) != STRSXP) error("FUN must be integer or character");
   const char * r = CHAR(STRING_ELT(x, 0)); // translateCharUTF8()
-  if(strcmp(r, "replace_NA") == 0) return 0;
+  if(strcmp(r, "replace_na") == 0) return 0;
+  if(strcmp(r, "na") == 0) return 0;
   if(strcmp(r, "replace_fill") == 0) return 1;
+  if(strcmp(r, "fill") == 0) return 1;
   if(strcmp(r, "replace") == 0) return 2;
   if(strcmp(r, "-") == 0) return 3;
   if(strcmp(r, "-+") == 0) return 4;
@@ -51,9 +53,11 @@ int TtI(SEXP x) {
   if(strcmp(r, "*") == 0) return 8;
   if(strcmp(r, "%%") == 0) return 9;
   if(strcmp(r, "-%%") == 0) return 10;
-  if(strcmp(r, "replace_na") == 0) return 0;
+  if(strcmp(r, "replace_NA") == 0) return 0;
+  if(strcmp(r, "NA") == 0) return 0;
   if(strcmp(r, "REPLACE_NA") == 0) return 0;
   if(strcmp(r, "REPLACE_FILL") == 0) return 1;
+  if(strcmp(r, "FILL") == 0) return 1;
   if(strcmp(r, "REPLACE") == 0) return 2;
   error("Unknown transformation: %s", r);
 }
@@ -121,13 +125,13 @@ SEXP ret1(SEXP x, SEXP xAG, SEXP g, int set) {
     }
     case STRSXP:
     {
-      SEXP *pout = STRING_PTR(out);
+      SEXP *pout = SEXPPTR(out);
       if(nog) {
         SEXP AG = asChar(xAG);
         #pragma omp simd
         for(int i = 0; i < l; ++i) pout[i] = AG;
       } else {
-        SEXP *AG = STRING_PTR(xAG)-1;
+        SEXP *AG = SEXPPTR(xAG)-1;
         #pragma omp simd
         for(int i = 0; i < l; ++i) pout[i] = AG[pg[i]];
       }
@@ -165,7 +169,7 @@ SEXP ret1(SEXP x, SEXP xAG, SEXP g, int set) {
   // 1 - x is classed (factor, date, time series), xAG is not classed. i.e. vector of fnobs, fmean etc.
   //    -> Sallow replacing, removing class and levels attributes from x, discard attributes of xAG (if any)
   //    -> or (if type matches i.e. double for date or time series), copy attributes of x unless x is a factor
-  // 2 - x is not classed, xAG is classed (factor, date, time series). - an unusual situation should not occurr - copy attributes of xAG, discard attributes of x
+  // 2 - x is not classed, xAG is classed (factor, date, time series). - an unusual situation should not occur - copy attributes of xAG, discard attributes of x
   // 3 - xAG and x are classed - same as above, keep attributes of xAG, discard attributes of x
   // 4 - neither x nor xAG are classed - preserve attributes of x, discard attributes of xAG (if any)
   //
@@ -233,13 +237,13 @@ SEXP ret2(SEXP x, SEXP xAG, SEXP g, int set) {
         break;
       }
       case STRSXP: {
-        SEXP *pout = STRING_PTR(out);
+        SEXP *pout = SEXPPTR(out);
         if(nog) {
           SEXP AG = asChar(xAG);
           #pragma omp simd
           for(int i = 0; i < l; ++i) pout[i] = ISNAN(px[i]) ? NA_STRING : AG;
         } else {
-          SEXP *AG = STRING_PTR(xAG)-1;
+          SEXP *AG = SEXPPTR(xAG)-1;
           #pragma omp simd
           for(int i = 0; i < l; ++i) pout[i] = ISNAN(px[i]) ? NA_STRING : AG[pg[i]];
         }
@@ -283,13 +287,13 @@ SEXP ret2(SEXP x, SEXP xAG, SEXP g, int set) {
         break;
       }
       case STRSXP: {
-        SEXP *pout = STRING_PTR(out);
+        SEXP *pout = SEXPPTR(out);
         if(nog) {
           SEXP AG = asChar(xAG);
           #pragma omp simd
           for(int i = 0; i < l; ++i) pout[i] = (px[i] == NA_INTEGER) ? NA_STRING : AG;
         } else {
-          SEXP *AG = STRING_PTR(xAG)-1;
+          SEXP *AG = SEXPPTR(xAG)-1;
           #pragma omp simd
           for(int i = 0; i < l; ++i) pout[i] = (px[i] == NA_INTEGER) ? NA_STRING : AG[pg[i]];
         }
@@ -302,7 +306,7 @@ SEXP ret2(SEXP x, SEXP xAG, SEXP g, int set) {
   }
   case STRSXP:
   {
-    SEXP *px = STRING_PTR(x);
+    SEXP *px = SEXPPTR(x);
     switch(txAG) {
       case REALSXP: {
         double *pout = REAL(out);
@@ -332,13 +336,13 @@ SEXP ret2(SEXP x, SEXP xAG, SEXP g, int set) {
         break;
       }
       case STRSXP: {
-        SEXP *pout = STRING_PTR(out);
+        SEXP *pout = SEXPPTR(out);
         if(nog) {
           SEXP AG = asChar(xAG);
           #pragma omp simd
           for(int i = 0; i < l; ++i) pout[i] = (px[i] == NA_STRING) ? NA_STRING : AG;
         } else {
-          SEXP *AG = STRING_PTR(xAG)-1;
+          SEXP *AG = SEXPPTR(xAG)-1;
           #pragma omp simd
           for(int i = 0; i < l; ++i) pout[i] = (px[i] == NA_STRING) ? NA_STRING : AG[pg[i]];
         }
@@ -444,7 +448,7 @@ SEXP ret0(SEXP x, SEXP xAG, SEXP g, int set) {
     }
     case STRSXP:
     {
-      SEXP *px = STRING_PTR(x), *pout = STRING_PTR(out);
+      SEXP *px = SEXPPTR(x), *pout = SEXPPTR(out);
       if(nog) {
         SEXP AG = asChar(xAG);
         #pragma omp simd
@@ -455,7 +459,7 @@ SEXP ret0(SEXP x, SEXP xAG, SEXP g, int set) {
           case LGLSXP:
           case INTSXP: error("Cannot replace missing values in string with numeric data");
           case STRSXP: {
-            SEXP *AG = STRING_PTR(xAG)-1;
+            SEXP *AG = SEXPPTR(xAG)-1;
             #pragma omp simd
             for(int i = 0; i < l; ++i) pout[i] = (px[i] == NA_STRING) ? AG[pg[i]] : px[i];
             break;
@@ -724,7 +728,7 @@ SEXP TRAlC(SEXP x, SEXP xAG, SEXP g, SEXP Rret, SEXP Rset) {
       break;
     }
     case STRSXP: {
-      SEXP *pAG = STRING_PTR(xAG);
+      SEXP *pAG = SEXPPTR(xAG);
       RETLOOPS(ScalarString(pAG[j]))
       break;
     }
@@ -810,7 +814,7 @@ SEXP TRAmC(SEXP x, SEXP xAG, SEXP g, SEXP Rret, SEXP Rset) {
           }
           case STRSXP:
           {
-            SEXP *pout = STRING_PTR(out), *pAG = STRING_PTR(xAG);
+            SEXP *pout = SEXPPTR(out), *pAG = SEXPPTR(xAG);
             if(nog) {
               for(int j = 0; j != col; ++j) {
                 int s = j * row, e = s + row;
@@ -879,7 +883,7 @@ SEXP TRAmC(SEXP x, SEXP xAG, SEXP g, SEXP Rret, SEXP Rset) {
               }
               case STRSXP:
               {
-                SEXP *pout = STRING_PTR(out), *pAG = STRING_PTR(xAG);
+                SEXP *pout = SEXPPTR(out), *pAG = SEXPPTR(xAG);
                 if(nog) {
                   for(int j = 0; j != col; ++j) {
                     int s = j * row, e = s + row;
@@ -947,7 +951,7 @@ SEXP TRAmC(SEXP x, SEXP xAG, SEXP g, SEXP Rret, SEXP Rset) {
               }
               case STRSXP:
               {
-                SEXP *pout = STRING_PTR(out), *pAG = STRING_PTR(xAG);
+                SEXP *pout = SEXPPTR(out), *pAG = SEXPPTR(xAG);
                 if(nog) {
                   for(int j = 0; j != col; ++j) {
                     int s = j * row, e = s + row;
@@ -971,7 +975,7 @@ SEXP TRAmC(SEXP x, SEXP xAG, SEXP g, SEXP Rret, SEXP Rset) {
           }
           case STRSXP:
           {
-            SEXP *px = STRING_PTR(x);
+            SEXP *px = SEXPPTR(x);
             switch(txAG) {
               case REALSXP:
               {
@@ -1014,7 +1018,7 @@ SEXP TRAmC(SEXP x, SEXP xAG, SEXP g, SEXP Rret, SEXP Rset) {
               }
               case STRSXP:
               {
-                SEXP *pout = STRING_PTR(out), *pAG = STRING_PTR(xAG);
+                SEXP *pout = SEXPPTR(out), *pAG = SEXPPTR(xAG);
                 if(nog) {
                   for(int j = 0; j != col; ++j) {
                     int s = j * row, e = s + row;
@@ -1161,14 +1165,14 @@ SEXP TRAmC(SEXP x, SEXP xAG, SEXP g, SEXP Rret, SEXP Rset) {
       }
       case STRSXP:
       {
-        SEXP *px = STRING_PTR(x), *pout = STRING_PTR(out);
+        SEXP *px = SEXPPTR(x), *pout = SEXPPTR(out);
         switch(txAG) {
         case REALSXP:
         case INTSXP:
         case LGLSXP: error("Cannot replace missing values in string with numeric data");
         case STRSXP:
         {
-          SEXP *pAG = STRING_PTR(xAG);
+          SEXP *pAG = SEXPPTR(xAG);
           if(nog) {
             for(int j = 0; j != col; ++j) {
               int s = j * row, e = s + row;
