@@ -175,9 +175,9 @@ SEXP dupVecIndex(SEXP x) {
     }
   } break;
   case STRSXP: {
-    const SEXP *restrict px = STRING_PTR(x);
+    const SEXP *restrict px = SEXPPTR(x);
     for (int i = 0; i != n; ++i) {
-      id = HASH(((intptr_t) px[i] & 0xffffffff), K);
+      id = HASH(((uintptr_t) px[i] & 0xffffffff), K);
       while(h[id]) {
         if(px[h[id]-1] == px[i]) {
           pans_i[i] = pans_i[h[id]-1]; // h[id];
@@ -338,13 +338,13 @@ SEXP dupVecIndexKeepNA(SEXP x) {
     }
   } break;
   case STRSXP: {
-    const SEXP *restrict px = STRING_PTR(x);
+    const SEXP *restrict px = SEXPPTR(x);
     for (int i = 0; i != n; ++i) {
       if(px[i] == NA_STRING) {
         pans_i[i] = NA_INTEGER;
         continue;
       }
-      id = HASH(((intptr_t) px[i] & 0xffffffff), K);
+      id = HASH(((uintptr_t) px[i] & 0xffffffff), K);
       while(h[id]) {
         if(px[h[id]-1] == px[i]) {
           pans_i[i] = pans_i[h[id]-1]; // h[id];
@@ -436,7 +436,7 @@ SEXP dupVecIndexTwoVectors(SEXP x, SEXP y) {
         case INTSXP: {
           const int *restrict px = INTEGER_RO(x), *restrict py = INTEGER_RO(y);
           for (int i = 0; i != n; ++i) {
-            id = HASH(px[i] + (64988430769U * (unsigned)py[i]), K); // Multiplication doesn't work here: too few unique values
+            id = HASH(px[i] + (64988430769U * py[i]), K); // Multiplication doesn't work here: too few unique values
             // Another large prime taken from https://oeis.org/wiki/Higher-order_prime_numbers
             while(h[id]) {
               hid = h[id]-1;
@@ -452,9 +452,9 @@ SEXP dupVecIndexTwoVectors(SEXP x, SEXP y) {
           }
         } break;
         case STRSXP: {
-          const SEXP *restrict px = STRING_PTR_RO(x), *restrict py = STRING_PTR_RO(y);
+          const SEXP *restrict px = SEXPPTR_RO(x), *restrict py = SEXPPTR_RO(y);
           for (int i = 0; i != n; ++i) {
-            id = HASH(64988430769U * ((intptr_t)px[i] & 0xffffffff) + ((intptr_t)py[i] & 0xffffffff), K); // Best combination it seems
+            id = HASH(64988430769U * ((uintptr_t)px[i] & 0xffffffff) + ((uintptr_t)py[i] & 0xffffffff), K); // Best combination it seems
             while(h[id]) {
               hid = h[id]-1;
               if(px[hid] == px[i] && py[hid] == py[i]) {
@@ -474,7 +474,7 @@ SEXP dupVecIndexTwoVectors(SEXP x, SEXP y) {
           // fill hash table with indices of 'table'
           for (int i = 0; i != n; ++i) {
             tpx.d = px[i]; tpy.d = py[i];
-            id = HASH((64988430769 * (tpx.u[0] + tpx.u[1])) + tpy.u[0] + tpy.u[1], K); // Best combination it seems
+            id = HASH((64988430769U * (tpx.u[0] + tpx.u[1])) + tpy.u[0] + tpy.u[1], K); // Best combination it seems
             while(h[id]) {
               hid = h[id]-1;
               if(REQUAL(px[hid], px[i]) && REQUAL(py[hid], py[i])) {
@@ -498,7 +498,7 @@ SEXP dupVecIndexTwoVectors(SEXP x, SEXP y) {
         union uno tpv;
         for (int i = 0; i != n; ++i) {
           tpv.d = pr[i];
-          id = HASH((64988430769U * (unsigned)pi[i]) + tpv.u[0] + tpv.u[1], K); // Best combination it seems
+          id = HASH((64988430769U * pi[i]) + tpv.u[0] + tpv.u[1], K); // Best combination it seems
           while(h[id]) {
             hid = h[id]-1;
             if(pi[hid] == pi[i] && REQUAL(pr[hid], pr[i])) {
@@ -513,12 +513,12 @@ SEXP dupVecIndexTwoVectors(SEXP x, SEXP y) {
         }
         // Second case: real and string
       } else if ((tx == REALSXP && ty == STRSXP) || (tx == STRSXP && ty == REALSXP)) {
-        const SEXP *restrict ps = STRING_PTR_RO(tx == STRSXP ? x : y);
+        const SEXP *restrict ps = SEXPPTR_RO(tx == STRSXP ? x : y);
         const double *restrict pr = REAL_RO(tx == REALSXP ? x : y);
         union uno tpv;
         for (int i = 0; i != n; ++i) {
           tpv.d = pr[i];
-          id = HASH((tpv.u[0] + tpv.u[1]) * ((intptr_t)ps[i] & 0xffffffff), K); // Best combination it seems
+          id = HASH((tpv.u[0] + tpv.u[1]) * ((uintptr_t)ps[i] & 0xffffffff), K); // Best combination it seems
           while(h[id]) {
             hid = h[id]-1;
             if(REQUAL(pr[hid], pr[i]) && ps[hid] == ps[i]) { // Seems comparing reals is faster..
@@ -534,9 +534,9 @@ SEXP dupVecIndexTwoVectors(SEXP x, SEXP y) {
         // Third case: integer and string
       } else if((tx == INTSXP && ty == STRSXP) || (tx == STRSXP && ty == INTSXP)) {
         const int *restrict pi = INTEGER_RO(tx == INTSXP ? x : y);
-        const SEXP *restrict ps = STRING_PTR_RO(tx == STRSXP ? x : y);
+        const SEXP *restrict ps = SEXPPTR_RO(tx == STRSXP ? x : y);
         for (int i = 0; i != n; ++i) {
-          id = HASH(pi[i] * ((intptr_t)ps[i] & 0xffffffff), K); // This is the fastest safe option
+          id = HASH(pi[i] * ((uintptr_t)ps[i] & 0xffffffff), K);
           while(h[id]) {
             hid = h[id]-1;
             if(pi[hid] == pi[i] && ps[hid] == ps[i]) {
@@ -635,8 +635,8 @@ int dupVecSecond(int *restrict pidx, int *restrict pans_i, SEXP x, const int n,
   case INTSXP: {
     const int *restrict px = INTEGER(x);
     const unsigned int mult = (M-1) / ng; // -1 because C is zero indexed
-    for (int i = 0; i != n; ++i) {
-      id = ((unsigned)pidx[i]*mult) ^ HASH(px[i], K); // HASH((unsigned)px[i] * (unsigned)pidx[i], K) + pidx[i]; // Need multiplication here instead of bitwise, see your benchmark with 100 mio. obs where second group is just sample.int(1e4, 1e8, T), there bitwise is very slow!!
+    for (int i = 0; i != n; ++i) { // Check this... DATA group_by lon, lat main_cat, main_tag, main_tag_value: main_tag is the issue..
+      id = (pidx[i]*mult) ^ HASH(px[i], K); // HASH((unsigned)px[i] * (unsigned)pidx[i], K) + pidx[i]; // Need multiplication here instead of bitwise, see your benchmark with 100 mio. obs where second group is just sample.int(1e4, 1e8, T), there bitwise is very slow!!
       while(h[id]) {  // However multiplication causes signed integer overflow... UBSAN error.
         hid = h[id]-1;
         if(pidx[hid] == pidx[i] && px[hid] == px[i]) { // Usually pidx has more distinct values...
@@ -656,7 +656,7 @@ int dupVecSecond(int *restrict pidx, int *restrict pans_i, SEXP x, const int n,
     union uno tpv;
     for (int i = 0; i != n; ++i) {
       tpv.d = px[i]; // R_IsNA(px[i]) ? NA_REAL : (R_IsNaN(px[i]) ? R_NaN :px[i]);
-      id = ((unsigned)pidx[i]*mult) ^ HASH(tpv.u[0] + tpv.u[1], K); // HASH((tpv.u[0] + tpv.u[1]) ^ pidx[i], K) + pidx[i]; // Note: This is much faster than just adding pidx[i] to the hash value...
+      id = (pidx[i]*mult) ^ HASH(tpv.u[0] + tpv.u[1], K); // HASH((tpv.u[0] + tpv.u[1]) ^ pidx[i], K) + pidx[i]; // Note: This is much faster than just adding pidx[i] to the hash value...
       while(h[id]) { // Problem: This value might be seen before, but not in combination with that pidx value...
         hid = h[id]-1; // The issue here is that REQUAL(px[hid], px[i]) could be true but pidx[hid] == pidx[i] fails, although the same combination of px and pidx could be seen earlier before...
         if(pidx[hid] == pidx[i] && REQUAL(px[hid], px[i])) {
@@ -687,7 +687,7 @@ int dupVecSecond(int *restrict pidx, int *restrict pans_i, SEXP x, const int n,
       u = tpv.u[0] ^ tpv.u[1];
       tpv.d = tmp.i;
       u ^= tpv.u[0] ^ tpv.u[1];
-      id = ((unsigned)pidx[i]*mult) ^ HASH(u, K);
+      id = (pidx[i]*mult) ^ HASH(u, K);
       while(h[id]) {
         hid = h[id]-1;
         if(pidx[hid] == pidx[i] && CEQUAL(px[hid], px[i])) {
@@ -702,10 +702,10 @@ int dupVecSecond(int *restrict pidx, int *restrict pans_i, SEXP x, const int n,
     }
   } break;
   case STRSXP: {
-    const SEXP *restrict px = STRING_PTR(x);
+    const SEXP *restrict px = SEXPPTR(x);
     const unsigned int mult = (M-1) / ng; // -1 because C is zero indexed
     for (int i = 0; i != n; ++i) {
-      id = ((unsigned)pidx[i]*mult) ^ HASH(((intptr_t) px[i] & 0xffffffff), K); // HASH(((intptr_t) px[i] & 0xffffffff) ^ pidx[i], K) + pidx[i];
+      id = (pidx[i]*mult) ^ HASH(((uintptr_t) px[i] & 0xffffffff), K); // HASH(((uintptr_t) px[i] & 0xffffffff) ^ pidx[i], K) + pidx[i];
       while(h[id]) {
         hid = h[id]-1;
         if(pidx[hid] == pidx[i] && px[hid] == px[i]) {
@@ -984,9 +984,9 @@ SEXP funiqueC(SEXP x) {
     for(int i = 0; i != g; ++i) pres[i] = px[st[i]];
   } break;
   case STRSXP: {
-    const SEXP *restrict px = STRING_PTR(x);
+    const SEXP *restrict px = SEXPPTR(x);
     for (int i = 0; i != n; ++i) {
-      id = HASH(((intptr_t) px[i] & 0xffffffff), K);
+      id = HASH(((uintptr_t) px[i] & 0xffffffff), K);
       while(h[id]) {
         if(px[h[id]-1] == px[i]) goto sbl;
         if(++id >= M) id = 0; // # nocov
@@ -1001,7 +1001,7 @@ SEXP funiqueC(SEXP x) {
       return x;
     }
     PROTECT(res = allocVector(STRSXP, g)); ++nprotect;
-    SEXP *restrict pres = STRING_PTR(res);
+    SEXP *restrict pres = SEXPPTR(res);
     for(int i = 0; i != g; ++i) pres[i] = px[st[i]];
   } break;
   }
