@@ -1,4 +1,5 @@
 #include "collapse_c.h"
+#include "base_radixsort.h"
 
 SEXP Cna_rm(SEXP x) {
   const int n = LENGTH(x);
@@ -31,11 +32,11 @@ SEXP Cna_rm(SEXP x) {
     return out;
   }
   case STRSXP: {
-    const SEXP *xd = STRING_PTR(x);
+    const SEXP *xd = SEXPPTR(x);
     for (int i = 0; i != n; ++i) if(xd[i] == NA_STRING) ++k;
     if(k == 0) return x;
     SEXP out = PROTECT(allocVector(STRSXP, n - k));
-    SEXP *pout = STRING_PTR(out);
+    SEXP *pout = SEXPPTR(out);
     k = 0;
     for (int i = 0; i != n; ++i) if(xd[i] != NA_STRING) pout[k++] = xd[i];
     copyMostAttrib(x, out);
@@ -60,7 +61,7 @@ SEXP Cna_rm(SEXP x) {
 
 // Helper function to find a single string in factor levels
 int fchmatch(SEXP x, SEXP val, int nomatch) {
-  const SEXP *px = STRING_PTR(x), v = asChar(val);
+  const SEXP *px = SEXPPTR(x), v = asChar(val);
   for(int i = 0, l = length(x); i != l; ++i) if(px[i] == v) return i + 1;
   return nomatch;
 }
@@ -100,13 +101,17 @@ if(length(val) == n && n > 1) {
   {
     const double *px = REAL(x);
     const double *pv = REAL(val);
-    WHICHVLOOPLX
+    if(invert) {
+      for(int i = 0; i != n; ++i) if(px[i] != pv[i] && (NISNAN(px[i]) || NISNAN(pv[i]))) buf[j++] = i+1;
+    } else {
+      for(int i = 0; i != n; ++i) if(px[i] == pv[i] || (ISNAN(px[i]) && ISNAN(pv[i]))) buf[j++] = i+1;
+    }
     break;
   }
   case STRSXP:
   {
-    const SEXP *px = STRING_PTR(x);
-    const SEXP *pv = STRING_PTR(val);
+    const SEXP *px = SEXPPTR(x);
+    const SEXP *pv = SEXPPTR(val);
     WHICHVLOOPLX
     break;
   }
@@ -151,7 +156,7 @@ if(length(val) == n && n > 1) {
   }
   case STRSXP:
   {
-    const SEXP *px = STRING_PTR(x);
+    const SEXP *px = SEXPPTR(x);
     const SEXP v = PROTECT(asChar(val));
     WHICHVLOOP
     UNPROTECT(1);
@@ -212,7 +217,7 @@ case REALSXP:
 }
 case STRSXP:
 {
-  const SEXP *px = STRING_PTR(x);
+  const SEXP *px = SEXPPTR(x);
   const SEXP v = asChar(val);
   ALLANYVLOOP
   break;
@@ -260,13 +265,15 @@ SEXP setcopyv(SEXP x, SEXP val, SEXP rep, SEXP Rinvert, SEXP Rset, SEXP Rind1) {
     if(lr != 1 && lr != n) error("If length(v) == 1, length(r) must be 1 or length(x)");
   }
 
-  if(lr > 1 && tr != tx) { // lr == n &&
+  if(tr != tx) { // lr == n &&
     if(!((tx == INTSXP && tr == LGLSXP) || (tx == LGLSXP && tr == INTSXP))) {
-      // PROTECT_INDEX ipx;
-      // PROTECT_WITH_INDEX(rep = coerceVector(rep, tx), &ipx);
-      tr = tx;
-      rep = PROTECT(coerceVector(rep, tx));
-      ++nprotect;
+      if(tr > tx && !(lr == 1 && tx == INTSXP && tr == REALSXP && REAL_ELT(rep, 0) == (int)REAL_ELT(rep, 0)))
+         warning("Type of R (%s) is larger than X (%s) and thus coerced. This incurs loss of information, such as digits of real numbers being truncated upon coercion to integer. To avoid this, make sure X has a larger type than R: character > double > integer > logical.", type2char(tr), type2char(tx));
+      if(lr > 1) {
+        tr = tx;
+        rep = PROTECT(coerceVector(rep, tx));
+        ++nprotect;
+      }
     } // error("typeof(x) needs to match typeof(r)");
   }
 
@@ -381,7 +388,7 @@ SEXP setcopyv(SEXP x, SEXP val, SEXP rep, SEXP Rinvert, SEXP Rset, SEXP Rind1) {
   }
   case STRSXP:
   {
-    SEXP *restrict px = set ? STRING_PTR(x) : STRING_PTR(ans);
+    SEXP *restrict px = set ? SEXPPTR(x) : SEXPPTR(ans);
     if(lv == 1 && ind1 == 0) {
       const SEXP v = PROTECT(asChar(val));
       if(lr == 1) {
@@ -389,7 +396,7 @@ SEXP setcopyv(SEXP x, SEXP val, SEXP rep, SEXP Rinvert, SEXP Rset, SEXP Rind1) {
         setcopyvLOOP(r)
         UNPROTECT(1);
       } else {
-        const SEXP *restrict pr = STRING_PTR(rep);
+        const SEXP *restrict pr = SEXPPTR(rep);
         setcopyvLOOP(pr[i])
       }
       UNPROTECT(1);
@@ -400,7 +407,7 @@ SEXP setcopyv(SEXP x, SEXP val, SEXP rep, SEXP Rinvert, SEXP Rset, SEXP Rind1) {
         setcopyvLOOPLVEC1
         UNPROTECT(1);
       } else {
-        const SEXP *restrict pr = STRING_PTR(rep);
+        const SEXP *restrict pr = SEXPPTR(rep);
         setcopyvLOOPLVEC
       }
     }
@@ -641,6 +648,217 @@ SEXP setop(SEXP x, SEXP val, SEXP op, SEXP roww) {
   return setop_core(x, val, op, roww);
 }
 
+
+SEXP replace_outliers(SEXP x, SEXP limits, SEXP value, SEXP single_limit, SEXP set)  {
+  const int ll = length(limits), sl = asInteger(single_limit), l = length(x), setl = asLogical(set);
+  int nprotect = setl == 0;
+  if(ll != 1 && ll != 2) error("'limits' must be length 1 or 2. You supplied limits length %d", ll);
+
+  int clip = 0;
+  if(TYPEOF(value) == STRSXP && strcmp(CHAR(STRING_ELT(value, 0)), "clip") == 0) {
+    value = limits;
+    clip = 1;
+  }
+
+  SEXP res = setl ? x : PROTECT(allocVector(TYPEOF(x), l));
+
+  switch(TYPEOF(x)) {
+    case INTSXP: {
+      if(TYPEOF(limits) != INTSXP) {
+        PROTECT(limits = coerceVector(limits, INTSXP)); ++nprotect;
+      }
+      int *px = INTEGER(x), *pres = INTEGER(res), val = asInteger(value);
+      if(ll == 1) {
+        if(sl == 2 || sl == 3) {
+          int l1 = INTEGER(limits)[0];
+          if(sl == 2) { // minimum
+            #pragma omp simd
+            for (int i = 0; i < l; ++i) pres[i] = px[i] < l1 && px[i] != NA_INTEGER ? val : px[i];
+          } else { // maximum
+            #pragma omp simd
+            for (int i = 0; i < l; ++i) pres[i] = px[i] > l1 ? val : px[i];
+          }
+        }
+      } else { // two-sided
+        int l1 = INTEGER(limits)[0], l2 = INTEGER(limits)[1];
+        if(clip) {
+          #pragma omp simd
+          for (int i = 0; i < l; ++i) pres[i] = px[i] > l2 ? l2 : px[i] < l1 && px[i] != NA_INTEGER ? l1 : px[i];
+        } else {
+          #pragma omp simd
+          for (int i = 0; i < l; ++i) pres[i] = px[i] > l2 || (px[i] < l1 && px[i] != NA_INTEGER) ? val : px[i];
+        }
+      }
+      break;
+    }
+    case REALSXP: {
+      if(TYPEOF(limits) != REALSXP) {
+        PROTECT(limits = coerceVector(limits, REALSXP)); ++nprotect;
+      }
+      double *px = REAL(x), *pres = REAL(res), val = asReal(value);
+      if(ll == 1) {
+        if(sl == 2 || sl == 3) {
+          double l1 = REAL(limits)[0];
+          if(sl == 2) { // minimum
+            #pragma omp simd
+            for (int i = 0; i < l; ++i) pres[i] = px[i] < l1 ? val : px[i];
+          } else { // maximum
+            #pragma omp simd
+            for (int i = 0; i < l; ++i) pres[i] = px[i] > l1 ? val : px[i];
+          }
+        }
+      } else { // two-sided
+        double l1 = REAL(limits)[0], l2 = REAL(limits)[1];
+        if(clip) {
+          #pragma omp simd
+          for (int i = 0; i < l; ++i) pres[i] = px[i] > l2 ? l2 : px[i] < l1 ? l1 : px[i];
+        } else {
+          #pragma omp simd
+          for (int i = 0; i < l; ++i) pres[i] = px[i] > l2 || px[i] < l1 ? val : px[i];
+        }
+      }
+      break;
+    }
+    default: error("Unsupported type '%s'", type2char(TYPEOF(x)));
+  }
+  if(setl == 0) SHALLOW_DUPLICATE_ATTRIB(res, x);
+  UNPROTECT(nprotect);
+  return res;
+}
+
+SEXP na_locf(SEXP x, SEXP Rset) {
+  int n = length(x), copy = asLogical(Rset) == 0;
+  if(isMatrix(x)) warning("na_locf() does not (yet) have explicit support for matrices, i.e., it treats a matrix as a single vector. Use dapply(M, na_locf) if column-wise processing is desired");
+  if(copy) x = PROTECT(shallow_duplicate(x));
+
+  switch (TYPEOF(x)) {
+  case INTSXP:
+  case LGLSXP:
+  {
+    int *data = INTEGER(x);
+    int last = data[0];
+    for (int i = 0; i < n; i++) {
+      if (data[i] == NA_INTEGER) {
+        data[i] = last;
+      } else {
+        last = data[i];
+      }
+    }
+    break;
+  }
+  case REALSXP:
+  {
+    double *data = REAL(x);
+    double last = data[0];
+    for (int i = 0; i < n; i++) {
+      if (ISNAN(data[i])) {
+        data[i] = last;
+      } else {
+        last = data[i];
+      }
+    }
+    break;
+  }
+  case STRSXP:
+  {
+    SEXP *data = SEXPPTR(x);
+    SEXP last = data[0];
+    for (int i = 0; i < n; i++) {
+      if (data[i] == NA_STRING) {
+        data[i] = last;
+      } else {
+        last = data[i];
+      }
+    }
+    break;
+  }
+  case VECSXP:
+  {
+    const SEXP *data = SEXPPTR_RO(x);
+    SEXP last = data[0];
+    for (int i = 0; i < n; i++) {
+      if (length(data[i]) == 0) {
+        SET_VECTOR_ELT(x, i, last);
+      } else {
+        last = data[i];
+      }
+    }
+    break;
+  }
+  default:
+    error("na_locf() does not support type '%s'", type2char(TYPEOF(x)));
+  }
+  UNPROTECT(copy);
+  return x;
+}
+
+SEXP na_focb(SEXP x, SEXP Rset) {
+  int n = length(x), copy = asLogical(Rset) == 0;
+  if(isMatrix(x)) warning("na_focb() does not (yet) have explicit support for matrices, i.e., it treats a matrix as a single vector. Use dapply(M, na_focb) if column-wise processing is desired");
+  if(copy) x = PROTECT(shallow_duplicate(x));
+
+  switch (TYPEOF(x)) {
+  case INTSXP:
+  case LGLSXP:
+  {
+    int *data = INTEGER(x);
+    int last = data[0];
+    for (int i = n; i--; ) {
+      if (data[i] == NA_INTEGER) {
+        data[i] = last;
+      } else {
+        last = data[i];
+      }
+    }
+    break;
+  }
+  case REALSXP:
+  {
+    double *data = REAL(x);
+    double last = data[0];
+    for (int i = n; i--; ) {
+      if (ISNAN(data[i])) {
+        data[i] = last;
+      } else {
+        last = data[i];
+      }
+    }
+    break;
+  }
+  case STRSXP:
+  {
+    SEXP *data = SEXPPTR(x);
+    SEXP last = data[0];
+    for (int i = n; i--; ) {
+      if (data[i] == NA_STRING) {
+        data[i] = last;
+      } else {
+        last = data[i];
+      }
+    }
+    break;
+  }
+  case VECSXP:
+  {
+    const SEXP *data = SEXPPTR_RO(x);
+    SEXP last = data[0];
+    for (int i = n; i--; ) {
+      if (length(data[i]) == 0) {
+        SET_VECTOR_ELT(x, i, last);
+      } else {
+        last = data[i];
+      }
+    }
+    break;
+  }
+  default:
+    error("na_focb() does not support type '%s'", type2char(TYPEOF(x)));
+  }
+  UNPROTECT(copy);
+  return x;
+}
+
+
 SEXP vtypes(SEXP x, SEXP isnum) {
   int tx = TYPEOF(x);
   if(tx != VECSXP) return ScalarInteger(tx);
@@ -660,7 +878,7 @@ SEXP vtypes(SEXP x, SEXP isnum) {
         is_num = 0;
         tci = TYPEOF(px[i]);
         tnum = tci == INTSXP || tci == REALSXP;
-        if(tnum) is_num = inherits(px[i], "integer") || inherits(px[i], "numeric") || inherits(px[i], "ts") || inherits(px[i], "units");
+        if(tnum) is_num = inherits(px[i], "integer") || inherits(px[i], "numeric") || inherits(px[i], "ts") || inherits(px[i], "units") || inherits(px[i], "integer64");
         pans[i] = tnum && is_num;
       }
       // for(int i = 0; i != n; ++i) {
@@ -673,24 +891,24 @@ SEXP vtypes(SEXP x, SEXP isnum) {
         tci = TYPEOF(px[i]);
         tnum = tci == INTSXP || tci == REALSXP;
         is_num = tnum && OBJECT(px[i]) == 0;
-        if(tnum && !is_num) is_num = inherits(px[i], "ts") || inherits(px[i], "units");
+        if(tnum && !is_num) is_num = inherits(px[i], "ts") || inherits(px[i], "units") || inherits(px[i], "integer64");
         pans[i] = is_num;
       }
     }
-    SET_TYPEOF(ans, LGLSXP);
+    SETTOF(ans, LGLSXP);
     break;
     }
   case 2: // is.factor
     for(int i = 0; i != n; ++i) pans[i] = (int)isFactor(px[i]);
-    SET_TYPEOF(ans, LGLSXP);
+    SETTOF(ans, LGLSXP);
     break;
   case 3: // is.list, needed for list processing functions
     for(int i = 0; i != n; ++i) pans[i] = TYPEOF(px[i]) == VECSXP;
-    SET_TYPEOF(ans, LGLSXP);
+    SETTOF(ans, LGLSXP);
     break;
   case 4: // is.sublist, needed for list processing functions
     for(int i = 0; i != n; ++i) pans[i] = TYPEOF(px[i]) == VECSXP && !isFrame(px[i]);
-    SET_TYPEOF(ans, LGLSXP);
+    SETTOF(ans, LGLSXP);
     break;
   case 7: // is.atomic(x), needed in atomic_elem()
     // is.atomic: do_is with op = 200:  https://github.com/wch/r-source/blob/9f9033e193071f256e21a181cb053cba983ed4a9/src/main/coerce.c
@@ -710,7 +928,7 @@ SEXP vtypes(SEXP x, SEXP isnum) {
         pans[i] = 0;
       }
     }
-    SET_TYPEOF(ans, LGLSXP);
+    SETTOF(ans, LGLSXP);
     break;
   case 5: // is.atomic(x) || is.list(x), needed in reg_elem() and irreg_elem()
     for(int i = 0; i != n; ++i) {
@@ -732,7 +950,7 @@ SEXP vtypes(SEXP x, SEXP isnum) {
         pans[i] = 0;
       }
     }
-    SET_TYPEOF(ans, LGLSXP);
+    SETTOF(ans, LGLSXP);
     break;
   case 6:
     // Faster object type identification, needed in unlist2d:
@@ -787,8 +1005,8 @@ SEXP vlengths(SEXP x, SEXP usenam) {
 
 
 // faster version of base::range, which calls both min() and max()
-SEXP frange(SEXP x, SEXP Rnarm) {
-  int l = length(x), narm = asLogical(Rnarm), tx = TYPEOF(x);
+SEXP frange(SEXP x, SEXP Rnarm, SEXP Rfinite) {
+  int l = length(x), narm = asLogical(Rnarm), finite = asLogical(Rfinite), tx = TYPEOF(x);
 
   SEXP out = PROTECT(allocVector(tx, 2));
 
@@ -835,14 +1053,25 @@ SEXP frange(SEXP x, SEXP Rnarm) {
         break;
       }
       double min, max, tmp, *px = REAL(x);
-      if(narm) {
+      if(narm || finite) {
         int j = l-1;
-        while(ISNAN(px[j]) && j!=0) --j;
+        if(finite) while(!R_FINITE(px[j]) && j!=0) --j;
+        else while(ISNAN(px[j]) && j!=0) --j;
         min = max = px[j];
-        if(j != 0) for(int i = j; i--; ) {
-          tmp = px[i];
-          if(min > tmp) min = tmp;
-          if(max < tmp) max = tmp;
+        if(j != 0) {
+          if(finite) {
+            for(int i = j; i--; ) {
+              tmp = px[i];
+              if(min > tmp && tmp > R_NegInf) min = tmp;
+              if(max < tmp && tmp < R_PosInf) max = tmp;
+            }
+          } else {
+            for(int i = j; i--; ) {
+              tmp = px[i];
+              if(min > tmp) min = tmp;
+              if(max < tmp) max = tmp;
+            }
+          }
         }
       } else {
         min = max = px[0];
